from __future__ import annotations


class ArchiveProvider:
    '''
    Interface class that provides methods for creating or appending to
    an archive.
    '''
    mime_type_providers = dict()
    file_extension_providers = dict()

    def __init__(self, archive) -> None:
        '''
        Initialize the provider with the archive it should use.

        Paramaters:
            archive         handle to the current archive
        '''
        self.archive = archive

    def open(name: str) -> ArchiveProvider:
        '''
        Open an existing archive.

        Parameters:
            name            filename of the archive to open

        Returns:
            None
        '''
        raise NotImplementedError

    def create(name: str) -> ArchiveProvider:
        '''
        Create an existing archive.

        Parameters:
            name            filename of the archive to create

        Returns:
            None
        '''
        raise NotImplementedError

    def append_file(self, filename: str, archived_name: str) -> None:
        '''
        Add an already existing file to the archive.

        Parameters:
            filename        file that is added to the archive
            archived_name   filename within the archive

        Returns:
            None
        '''
        raise NotImplementedError

    def append_files(self, filename: str, archived_names: [str]) -> None:
        '''
        Add an already existing file multiple times to the same archive,
        but use different archive names specified as list.

        Parameters:
            filename        file that is added to the archive
            archived_names  list of file names within the archive

        Returns:
            None
        '''
        for name in archived_names:
            self.append_file(filename, name)

    def append_blob(self, blob: bytes, archived_name: str) -> None:
        '''
        Append a data blob to the archive.

        Parameters:
            blob            blob of bytes to append to the archive
            archived_name   file name within the archive

        Returns:
            None
        '''
        raise NotImplementedError

    def append_blobs(self, blob: bytes, archived_names: [str]) -> None:
        '''
        Append a data blob to the archive under multiple different archive names.

        Parameters:
            blob            blob of bytes to append to the archive
            archived_names  list of file names within the archvie

        Returns:
            None
        '''
        for name in archived_names:
            self.append_blob(blob, name)

    def append_symlink(self, target: str, archived_name: str) -> None:
        '''
        Append a symlink to the archive.

        Parameters:
            target          symlink target
            archived_name   file name within the archive

        Returns:
            None
        '''
        raise NotImplementedError

    def append_symlinks(self, target: str, archived_names: [str]) -> None:
        '''
        Append a symlink to the archive with several different archive names.

        Parameters:
            target          symlink target
            archived_names  list of file names within the archvie

        Returns:
            None
        '''
        for name in archived_names:
            self.append_symlink(target, name)

    def remove_files(name: str, archived_name: str) -> None:
        '''
        Remove matching files from the archive.

        Parameters:
            name            file system path to the archive
            archived_name   filenames to remove from the archive

        Returns:
            None
        '''
        raise NotImplementedError

    def list_archive(name: str) -> None:
        '''
        Print a list of files contained within the archive.

        Parameters:
            name            file system path to the archive

        Returns:
            None
        '''
        raise NotImplementedError

    def clear_archive(name: str, payload: str) -> None:
        '''
        Clear the specified archive from path traversal sequences.

        Parameters:
            name            file system path of the archive
            payload         path traversal payload to look for

        Returns:
            None
        '''
        raise NotImplementedError

    def close_archive(self) -> None:
        '''
        Close the archive.

        Parameters:
            None

        Returns:
            None
        '''
        self.archive.close()

    def register_provider_ext(provider: ArchiveProvider, ext: str) -> None:
        '''
        Register an archive provider for the specified extension.

        Parameters:
            provider            ArchiveProvider to register
            ext                 file extension to register for

        Returns:
            None
        '''
        if ArchiveProvider.file_extension_providers.get(ext) is not None:
            raise ValueError(f"Provider for extension {ext} was already registered!")

        ArchiveProvider.file_extension_providers[ext] = provider

    def get_provider_ext(ext: str) -> ArchiveProvider:
        '''
        Return the ArchiveProvider that was registered for the specified file extension.

        Parameters:
            mime                file extension to get the provider for

        Returns:
            ArchiveProvider for the requested file extension
        '''
        return ArchiveProvider.file_extension_providers.get(ext)

    def register_provider_mime(provider: ArchiveProvider, mime: str) -> None:
        '''
        Register an archive provider for the specified mime type.

        Parameters:
            provider            ArchiveProvider to register
            mime                mime type to register the provider for

        Returns:
            None
        '''
        if ArchiveProvider.mime_type_providers.get(mime) is not None:
            raise ValueError(f"Provider for mime type {mime} was already registered!")

        ArchiveProvider.mime_type_providers[mime] = provider

    def get_provider_mime(mime: str) -> ArchiveProvider:
        '''
        Return the ArchiveProvider that was registered for the specified mime type.

        Parameters:
            mime                mime type to get the provider for

        Returns:
            ArchiveProvider for the requested mime type
        '''
        return ArchiveProvider.mime_type_providers.get(mime)



from __future__ import annotations

import zipfile
import fnmatch
import warnings
from pathlib import Path
from slipit.archive_provider import ArchiveProvider


class ZipProvider(ArchiveProvider):
    '''
    ArchiveProvider for zip files.
    '''

    def open(name: str) -> ArchiveProvider:
        '''
        Open the specified archive.

        Parameters:
            name            file system path of the archive

        Returns:
            ArchiveProvider for the opened archive
        '''
        zip_file = zipfile.ZipFile(name, 'a')
        return ZipProvider(zip_file)

    def create(name: str) -> ArchiveProvider:
        '''
        Create the specified archive.

        Parameters:
            name            file system path of the archive

        Returns:
            ArchiveProvider for the created archive
        '''
        zip_file = zipfile.ZipFile(name, 'w')
        return ZipProvider(zip_file)

    def append_file(self, filename: str, archived_name: str) -> None:
        '''
        Append a file to the archive.

        Parameters:
            filename            file system path to read the file from
            archived_name       file name within the archive

        Returns:
            None
        '''
        with warnings.catch_warnings():
            warnings.filterwarnings('ignore', message='Duplicate name')
            self.archive.write(filename, archived_name)

    def append_blob(self, blob: bytes, archived_name: str) -> None:
        '''
        Append a data blob to the archive

        Parameters:
            blob                blob of bytes to append to the archive
            archived_name       file name within the archive

        Returns:
            None
        '''
        with warnings.catch_warnings():
            warnings.filterwarnings('ignore', message='Duplicate name')
            self.archive.writestr(archived_name, blob)

    def list_archive(name: str) -> None:
        '''
        Print a list of files contained within the archive.

        Parameters:
            name            file system path of the archive

        Returns:
            None
        '''
        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with zipfile.ZipFile(name, 'r') as zip_file:
            zip_file.printdir()

    def remove_files(name: str, archived_name: str) -> None:
        '''
        Remove files matching the specified filename from the archive.

        Parameters:
            name            file system path of the archive
            archived_name   filename to match files against

        Returns:
            None
        '''
        ZipProvider.remove_from_archive(name, archived_name, True)

    def clear_archive(name: str, payload: str) -> None:
        '''
        Clear the specified archive from path traversal sequences.

        Parameters:
            name            file system path of the archive
            payload         path traversal payload to look for

        Returns:
            None
        '''
        ZipProvider.remove_from_archive(name, payload, False)

    def remove_from_archive(name: str, payload: str, use_fnmatch: bool) -> None:
        '''
        Remove all files matching the specified payload from the archive.
        The payload is either matched by using an 'in' statement on the
        archive filenames (use_fnmatch=False) or by using fnmatch
        (use_fnmatch=True).

        Parameters:
            name            file system path of the archive
            payload         payload to match filenames against
            fnmatch         whether to use fnmatch for matching

        Returns:
            None
        '''
        member_content_map = dict()

        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with zipfile.ZipFile(name, 'r') as zip_file:

            for member in zip_file.infolist():

                if member.is_dir():
                    member_content_map[member] = ""

                else:
                    content = zip_file.open(member.filename).read()
                    member_content_map[member] = content

        with zipfile.ZipFile(name, 'w') as output:

            for member, content in member_content_map.items():

                if use_fnmatch and not fnmatch.fnmatch(member.filename, payload):
                    output.writestr(member, content)

                if not use_fnmatch and payload not in member.filename:
                    output.writestr(member, content)


for ext in ['.zip', '.jar', '.doc', '.docx']:
    ArchiveProvider.register_provider_ext(ZipProvider, ext)

for mime in ['application/zip', 'multipart/x-zip', 'application/x-zip-compressed', 'application/msword',
             'application/vnd.openxmlformats-officedocument.wordprocessingml.document']:
    ArchiveProvider.register_provider_mime(ZipProvider, mime)



from .archive_provider import ArchiveProvider
from .provider.tar_provider import TarProvider
from .provider.zip_provider import ZipProvider
from .provider.compressed_tar_provider import GZipProvider, BZip2Provider

name = 'slipit'



from __future__ import annotations

import io
import fnmatch
import tarfile
from pathlib import Path
from slipit.archive_provider import ArchiveProvider
from slipit.provider.tar_provider import TarProvider


class CompressedTarProvider(TarProvider):
    '''
    ArchiveProvider for tar files.
    '''

    def open(name: str, alg: str = 'gz') -> TarProvider:
        '''
        Open the specified archive.

        Parameters:
            name            file system path of the archive
            alg             compression algorithm

        Returns:
            ArchiveProvider for the opened archive
        '''
        member_content_map = dict()

        if Path(name).is_file():

            with tarfile.open(name, f'r:{alg}') as tar_file:

                for member in tar_file.getmembers():

                    if member.isfile():
                        content = tar_file.extractfile(member).read(member.size)
                        member_content_map[member] = io.BytesIO(content)

                    else:
                        member_content_map[member] = None

        output = tarfile.open(name, f'w:{alg}')

        for member, content in member_content_map.items():
            output.addfile(member, content)

        return TarProvider(output)

    def create(name: str, alg: str) -> TarProvider:
        '''
        Create the specified archive.

        Parameters:
            name            file system path of the archive
            alg             compression algorithm

        Returns:
            ArchiveProvider for the created archive
        '''
        tar_file = tarfile.open(name, f'w:{alg}')
        return TarProvider(tar_file)

    def list_archive(name: str, alg: str = 'gz') -> None:
        '''
        Print a list of the archives content to stdout.

        Parameters:
            name            file system path of the archive
            alg             compression algorithm

        Returns:
            None
        '''
        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with tarfile.open(name, f'r:{alg}') as tar_file:
            tar_file.list()

    def remove_files(name: str, archived_name: str, alg: str = 'gz') -> None:
        '''
        Remove files matching the specified filename from the archive.

        Parameters:
            name            file system path of the archive
            archived_name   filename to match files against

        Returns:
            None
        '''
        CompressedTarProvider.remove_from_archive(name, archived_name, True, alg)

    def clear_archive(name: str, payload: str, alg: str = 'gz') -> None:
        '''
        Clear the specified archive from all path traversal sequences.

        Parameters:
            name            file system path of the archive
            payload         path traversal payload to look for
            alg             compression algorithm

        Returns:
            None
        '''
        CompressedTarProvider.remove_from_archive(name, payload, False, alg)

    def remove_from_archive(name: str, payload: str, use_fnmatch: bool, alg: str ='gz') -> None:
        '''
        Remove all files matching the specified payload from the archive.
        The payload is either matched by using an 'in' statement on the
        archive filenames (use_fnmatch=False) or by using fnmatch
        (use_fnmatch=True).

        Parameters:
            name            file system path of the archive
            payload         payload to match filenames against
            fnmatch         whether to use fnmatch for matching
            alg             compression algorithm

        Returns:
            None
        '''
        member_content_map = dict()

        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with tarfile.open(name, f'r:{alg}') as tar_file:

            for member in tar_file.getmembers():

                if member.isfile():
                    content = tar_file.extractfile(member).read(member.size)
                    member_content_map[member] = io.BytesIO(content)

                else:
                    member_content_map[member] = None

        with tarfile.open(name, f'w:{alg}') as output:

            for member, content in member_content_map.items():

                if use_fnmatch and not fnmatch.fnmatch(member.name, payload):
                    output.addfile(member, content)

                if not use_fnmatch and payload not in member.name:
                    output.addfile(member, content)


class GZipProvider(CompressedTarProvider):
    '''
    Archive provider for gzip compressed tar archives.
    '''
    def open(name: str) -> GZipProvider:
        '''
        '''
        return CompressedTarProvider.open(name, 'gz')

    def create(name: str) -> GZipProvider:
        '''
        '''
        return CompressedTarProvider.create(name, 'gz')

    def list_archive(name: str) -> None:
        '''
        '''
        return CompressedTarProvider.list_archive(name, 'gz')

    def remove_files(name: str, payload: str) -> None:
        '''
        '''
        return CompressedTarProvider.remove_files(name, payload, 'gz')

    def clear_archive(name: str, payload: str) -> None:
        '''
        '''
        return CompressedTarProvider.clear_archive(name, payload, 'gz')


class BZip2Provider(CompressedTarProvider):
    '''
    Archive provider for gzip compressed tar archives.
    '''

    def open(name: str) -> BZip2Provider:
        '''
        '''
        return CompressedTarProvider.open(name, 'bz2')

    def create(name: str) -> BZip2Provider:
        '''
        '''
        return CompressedTarProvider.create(name, 'bz2')

    def list_archive(name: str) -> BZip2Provider:
        '''
        '''
        return CompressedTarProvider.list_archive(name, 'bz2')

    def remove_files(name: str, payload: str) -> None:
        '''
        '''
        return CompressedTarProvider.remove_files(name, payload, 'bz2')

    def clear_archive(name: str, payload: str) -> BZip2Provider:
        '''
        '''
        return CompressedTarProvider.clear_archive(name, payload, 'bz2')


for ext in ['.gz', '.tgz']:
    ArchiveProvider.register_provider_ext(GZipProvider, ext)

for mime in ['application/gzip', 'application/x-gzip', 'application/x-gtar', 'application/x-tgz']:
    ArchiveProvider.register_provider_mime(GZipProvider, mime)

ArchiveProvider.register_provider_ext(BZip2Provider, '.bz2')
ArchiveProvider.register_provider_mime(BZip2Provider, 'application/x-bzip2')



from __future__ import annotations

import io
import tarfile
import fnmatch
from pathlib import Path
from slipit.archive_provider import ArchiveProvider


class TarProvider(ArchiveProvider):
    '''
    ArchiveProvider for tar files.
    '''
    def open(name: str) -> ArchiveProvider:
        '''
        Open the specified archive.

        Parameters:
            name            file system path of the archive

        Returns:
            ArchiveProvider for the opened archive
        '''
        tar_file = tarfile.open(name, 'a:')
        return TarProvider(tar_file)

    def create(name: str) -> ArchiveProvider:
        '''
        Create the specified archive.

        Parameters:
            name            file system path of the archive

        Returns:
            ArchiveProvider for the created archive
        '''
        tar_file = tarfile.open(name, 'w:')
        return TarProvider(tar_file)

    def append_file(self, filename: str, archived_name: str) -> None:
        '''
        Append a file to the archive.

        Parameters:
            filename            file system path to read the file from
            archived_name       file name within the archive

        Returns:
            None
        '''
        self.archive.add(filename, arcname=archived_name)

    def append_blob(self, blob: bytes, archived_name: str) -> None:
        '''
        Append a data blob to the archive.

        Parameters:
            blob                blob of bytes to append to the archive
            archived_name       file name within the archive

        Returns:
            None
        '''
        info = tarfile.TarInfo()

        info.type = tarfile.REGTYPE
        info.name = archived_name
        info.size = len(blob)

        file_like = io.BytesIO(blob)

        self.archive.addfile(info, file_like)

    def append_symlink(self, target: str, archived_name: str) -> None:
        '''
        Append a symlink to the archive.

        Parameters:
            target              symlink target
            archived_name       file name within the archive

        Returns:
            None
        '''
        info = tarfile.TarInfo()

        info.type = tarfile.SYMTYPE
        info.name = archived_name
        info.linkname = target

        self.archive.addfile(info, archived_name)

    def list_archive(name: str) -> None:
        '''
        Print a list of the archives content to stdout.

        Parameters:
            name            file system path of the archive

        Returns:
            None
        '''
        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with tarfile.open(name, 'r:') as tar_file:
            tar_file.list()

    def remove_files(name: str, archived_name: str) -> None:
        '''
        Remove files matching the specified filename from the archive.

        Parameters:
            name            file system path of the archive
            archived_name   filename to match files against

        Returns:
            None
        '''
        TarProvider.remove_from_archive(name, archived_name, True)

    def clear_archive(name: str, payload: str) -> None:
        '''
        Clear the specified archive from path traversal sequences.

        Parameters:
            name            file system path of the archive
            payload         path traversal payload to look for

        Returns:
            None
        '''
        TarProvider.remove_from_archive(name, payload, False)

    def remove_from_archive(name: str, payload: str, use_fnmatch: bool) -> None:
        '''
        Remove all files matching the specified payload from the archive.
        The payload is either matched by using an 'in' statement on the
        archive filenames (use_fnmatch=False) or by using fnmatch
        (use_fnmatch=True).

        Parameters:
            name            file system path of the archive
            payload         payload to match filenames against
            fnmatch         whether to use fnmatch for matching

        Returns:
            None
        '''
        member_content_map = dict()

        if not Path(name).is_file():
            raise FileNotFoundError(name)

        with tarfile.open(name, 'r:') as tar_file:

            for member in tar_file.getmembers():

                if member.isfile():
                    content = tar_file.extractfile(member).read(member.size)
                    member_content_map[member] = io.BytesIO(content)

                else:
                    member_content_map[member] = None

        with tarfile.open(name, 'w:') as output:

            for member, content in member_content_map.items():

                if use_fnmatch and not fnmatch.fnmatch(member.name, payload):
                    output.addfile(member, content)

                if not use_fnmatch and payload not in member.name:
                    output.addfile(member, content)


for ext in ['.tar']:
    ArchiveProvider.register_provider_ext(TarProvider, ext)

for mime in ['application/x-tar', 'application/tar']:
    ArchiveProvider.register_provider_mime(TarProvider, mime)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='slipit',
    version='1.0.0',
    url='https://github.com/usdAG/slipit',
    author='Tobias Neitzel (@qtc_de)',
    description='slipit - Utility for creating archives with path traversal elements',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
                        'python-magic',
                     ],
    packages=[
                'slipit',
                'slipit.provider'
             ],
    scripts=[
                'bin/slipit'
            ],
    classifiers=[
                    'Programming Language :: Python :: 3',
                    'Operating System :: Unix',
                    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                ],
)
